@@ -44,12 +44,12 @@
 
 typedef struct __GLXtextureFromPixmap __GLXtextureFromPixmap;
 struct __GLXtextureFromPixmap {
-    int (*bindTexImage)		(__GLXcontext *baseContext,
-				 int           buffer,
-				 __GLXpixmap  *pixmap);
-    int (*releaseTexImage)	(__GLXcontext *baseContext,
-				 int           buffer,
-				 __GLXpixmap  *pixmap);
+    int (*bindTexImage)		(__GLXcontext  *baseContext,
+				 int            buffer,
+				 __GLXdrawable *pixmap);
+    int (*releaseTexImage)	(__GLXcontext  *baseContext,
+				 int            buffer,
+				 __GLXdrawable *pixmap);
 };
 
 
@@ -85,14 +85,8 @@ struct __GLXcontext {
     ** Pointer to screen info data for this context.  This is set
     ** when the context is created.
     */
-    ScreenPtr pScreen;
     __GLXscreen *pGlxScreen;
 
-    /*
-    ** This context is created with respect to this visual.
-    */
-    VisualRec *pVisual;
-
     /*
     ** The XID of this context.
     */
@@ -103,11 +97,6 @@ struct __GLXcontext {
     */
     XID share_id;
 
-    /*
-    ** Visual id.
-    */
-    VisualID vid;
-
     /*
     ** screen number.
     */
@@ -128,11 +117,6 @@ struct __GLXcontext {
     */
     GLboolean isDirect;
 
-    /*
-    ** Window pending state
-    */
-    GLuint pendingState;
-
     /*
     ** This flag keeps track of whether there are unflushed GL commands.
     */
@@ -151,12 +135,6 @@ struct __GLXcontext {
     GLuint *selectBuf;
     GLint selectBufSize;	/* number of elements allocated */
 
-    /*
-    ** Set only if current drawable is a glx pixmap.
-    */
-    __GLXpixmap *drawPixmap;
-    __GLXpixmap *readPixmap;
-
     /*
     ** The drawable private this context is bound to
     */
